#!/usr/bin/perl

$ngcfile = ".NGC2000_0.txt";
$mfile = "m.txt";
$mdatafile = "mess_fr.dat";
$creffile = "dsos-cr.txt" ;
$pixfile = "pix_fr.txt";
$capfile = "cap.txt";
$mpixdir = "/messier/JpgSm/";
$mcapdir = "/messier/m/";

$NGCmax = 7840;
$ICmax = 5386;
$Mmax = 110;

$mnum = 0; # Initialization

%types = (
  "Gx",  "Galaxie",
  "OC",  "Amas ouvert",
  "Gb",  "Amas globulaire", 
  "Nb",  "N&eacute;buleuse brillante (&eacute;mission et/ou r&eacute;flexion)",
  "Pl",  "N&eacute;buleuse plan&eacute;taire",
  "C+N", "Amas associ&eacute; &agrave; une n&eacute;bulosit&eacute;",
  "Ast", "Ast&eacute;risme ou groupe de quelques &eacute;toiles",
  "Kt",  "Noeud ou r&eacute;gion n&eacute;buleuse dans une galaxie externe",
  "***", "Etoile triple",
  "D*",  "Etoile double",
  "D*?", "Etoile double (?)",
  "*",   "Etoile unique",
  "*?",  "Etoile unique (?)",
  "?",   "Objet de type incertain ou &agrave; l'existence non v&eacute;rifi&eacute;e,",
  "",    "Non identifi&eacute; &agrave; l'endroit indiqu&eacute;, ou de type inconnu,",
  "-",   "Objet mentionn&eacute; comme inexistant dans le catalogue RNGC (Sulentic and Tifft 1973),",
  "PD",  "Plaque photographique d&eacute;fectueuse ",
);

%constell = (
  "And", "Androm&egrave;de", # "Andromeda", 
  "Ant", "la Machine pneumatique", # "Antlia",  
  "Aps", "l'Oiseau de Paradis", # "Apus",  
  "Aqr", "le Verseau", # "Aquarius",
  "Aql", "l'Aigle", # "Aquila",
  "Ara", "l'Autel", # "Ara",
  "Ari", "le B&eacute;lier", # "Aries", 
  "Aur", "le Cocher", # "Auriga",
  "Boo", "le Bouvier", # "Bootes",
  "Cae", "le Burin", # "Caelum", 
  "Cam", "la Girafe", # "Camelopardalis", 
  "Cnc", "le Cancer", # "Cancer",
  "CVn", "les Chiens de chasse", # "Canes Venatici",
  "CMa", "le Grand Chien", # "Canis Major",
  "CMi", "le Petit Chien", # "Canis Minor",
  "Cap", "le Capricorne", # "Capricornus",
  "Car", "la Car&egrave;ne", # "Carina", 
  "Cas", "Cassiop&eacute;e", # "Cassiopeia",
  "Cen", "le Centaure", # "Centaurus",  
  "Cep", "C&eacute;ph&eacute;e", # "Cepheus",
  "Cet", "la Baleine", # "Cetus",
  "Cha", "le Cam&eacute;l&eacute;on", # "Chamaeleon",  
  "Cir", "le Compas", # "Circinus", 
  "Col", "la Colombe", # "Columba", 
  "Com", "la Chevelure de B&eacute;r&eacute;nice", # "Coma Berenices",  
  "CrA", "la Couronne Australe", # "Corona Australis", 
  "CrB", "la Couronne Bor&eacute;ale", # "Corona Borealis",  
  "Crv", "le Corbeau", # "Corvus", 
  "Crt", "la Coupe", # "Crater", 
  "Cru", "la Croix du Sud", # "Crux", 
  "Cyg", "le Cygne", # "Cygnus",
  "Del", "le Dauphin", # "Delphinus", 
  "Dor", "la Dorade", # "Dorado", 
  "Dra", "le Dragon", # "Draco",
  "Equ", "le Petit Cheval", # "Equuleus", 
  "Eri", "l'Eridan", # "Eridanus", 
  "For", "le Fourneau", # "Fornax", 
  "Gem", "les G&eacute;meaux", # "Gemini",
  "Gru", "la Grue", # "Grus", 
  "Her", "Hercule", # "Hercules",
  "Hor", "l'Horloge", # "Horologium", 
  "Hya", "l'Hydre Femelle", # "Hydra",
  "Hyi", "l'Hydre M&acirc;le", # "Hydrus",
  "Ind", "l'Indien", # "Indus", 
  "Lac", "le L&eacute;zard", # "Lacerta", 
  "Leo", "le Lion", # "Leo",
  "LMi", "le Petit Lion", # "Leo Minor",
  "Lep", "le Li&egrave;vre", # "Lepus",
  "Lib", "la Balance", # "Libra", 
  "Lup", "le Loup", # "Lupus", 
  "Lyn", "le Lynx", # "Lynx", 
  "Lyr", "la Lyre", # "Lyra",
  "Men", "la Table", # "Mensa", 
  "Mic", "le Microscope", # "Microscopium", 
  "Mon", "la Licorne", # "Monoceros",
  "Mus", "la Mouche", # "Musca", 
  "Nor", "la R&egrave;gle", # "Norma", 
  "Oct", "l'Octant", # "Octans", 
  "Oph", "Ophiuchus", # "Ophiuchus",
  "Ori", "Orion", # "Orion",
  "Pav", "le Paon", # "Pavo", 
  "Peg", "P&eacute;gase", # "Pegasus",
  "Per", "Pers&eacute;e", # "Perseus",
  "Phe", "le Ph&eacute;nix", # "Phoenix", 
  "Pic", "le Peintre", # "Pictor", 
  "Psc", "les Poissons", # "Pisces",
  "PsA", "le Poisson Austral", # "Piscis Austrinus", 
  "Pup", "la Poupe", # "Puppis",
  "Pyx", "la Boussole", # "Pyxis", 
  "Ret", "le R&eacute;ticule", # "Reticulum", 
  "Sge", "la Fl&egrave;che", # "Sagitta",
  "Sgr", "le Sagittaire", # "Sagittarius",
  "Sco", "le Scorpion", # "Scorpius",
  "Scl", "le Sculpteur", # "Sculptor",
  "Sct", "l'Ecu", # "Scutum",
  "Ser", "le Serpent", # "Serpens",
  "Sex", "le Sextant", # "Sextans", 
  "Tau", "le Taureau", # "Taurus",
  "Tel", "le T&eacute;lescope", # "Telescopium", 
  "Tri", "le Triangle", # "Triangulum",
  "TrA", "le Triangle Austral", # "Triangulum Australe",  
  "Tuc", "le Toucan", # "Tucana", 
  "UMa", "la Grande Ourse", # "Ursa Major",
  "UMi", "la Petite Ourse", # "Ursa Minor",
  "Vel", "les Voiles", # "Vela", 
  "Vir", "la Vierge", # "Virgo",
  "Vol", "le Poisson volant", # "Volans", 
  "Vul", "le Petit Renard", # "Vulpecula",
);

###########################################

$catnum = $ENV{QUERY_STRING};

if ( $catnum eq "HOMEPAGE" ) {
  open HOMEPAGE , "ngc_fr.html" ;
  @lines = <HOMEPAGE> ;
  close HOMEPAGE ;
  print "Content-type: text/html\n\n";
  print @lines ; 
  exit 0 ;
}

$catnum =~ s/\x2b/ /g; # "+" of  parameters -> spaces
$catnum =~ s/^[ ]*//;  # remove leading spaces

if ($catnum =~ /^i/i) { # IC catalog number
  $cat = "IC";
  $catnum =~ s/\D//g; $catnum = $catnum + 0;
  if ($catnum > $ICmax) {
    $errstr ="Num&eacute;ro IC ($catnum) hors limite (1-$ICmax).";
    $cat = "";
    $title = "Erreur NGC !";
    $catnum = 0;
  } else {
    $title = "IC " . $catnum;
    if ($catnum == 4725) { # M25, the only Messier with an IC number 
      $mnum = 25;
      $title .= " = M25";
    } elsif ($catnum == 4715) { # possibly M24, perhaps a second ..
      $mnum = 24;
      $title .= " = M24?";
    };
    $cnum = "I " . (" " x (4-length($catnum))) . $catnum;
    $catnum = "I" . (" " x (4-length($catnum))) . $catnum;
  };
} elsif ($catnum =~ /^[n\d]/i) { # NGC number 
  $cat = "NGC";
  $catnum =~ s/\D//g; $catnum = $catnum + 0;
  if ($catnum > $NGCmax) {
    $errstr ="Num&eacute;ro NGC ($catnum) hors limite (1-$NGCmax).";
    $cat = "";
    $title = "Erreur NGC !";
    $catnum = 0;
  } else {
    $cnum = "N " . (" " x (4-length($catnum))) . $catnum;
    $catnum = "N" . (" " x (4-length($catnum))) . $catnum;
    $mline = `grep "$catnum" $mfile`; # find possible Messier number
    if ($mline =~ /^M[ ]*(\d*) =/) {
      $mnum = $1;
    };
    $catnum =~ s/N/ /;
    $title = "NGC" . $catnum;
    if ($mnum) {
      $title .= " = M" . $mnum;
      if ($mnum == 24) {$title =~ s/=/in/};
    };
  };
} elsif ($catnum =~ /^m/i) { # Messier number
  $cat = "M";
  $catnum =~ s/\D//g; $catnum = $catnum + 0;
  if ($catnum > $Mmax) {
    $errstr ="Num&eacute;ro Messier ($catnum) hors limite (1-$Mmax).";
    $cat = "";
    $title = "Erreur NGC !";
    $catnum = 0;
  } else {
    $mnum = $catnum;
    $catnum = "M" . (" " x (3 - length($catnum))) . $catnum;
    $cnum = $catnum;
    $mline = `grep "$catnum" $mfile`;
    if ($mline =~ /.*([NI][ ]*\d*)/) {
      $catnum = $1;
      $catnum =~ s/N/ /;
      if ($mnum == 24) {
        $title = "M24 (= IC 4715?) avec NGC 6603";
      } else {
        $catnum =~ s/ /NGC /  || $catnum =~ s/I/IC /;
        $title = $catnum . " = M" . $mnum;
        $catnum =~ s/NGC / /  || $catnum =~ s/IC /I/;
      };
    } else { # M45, M40
      $title = "M" . $mnum;
      $catnum = 0; # No NGC or IC number 
    };
  };
} else { # erroneous input
  $cat = "";
  $title = "Erreur NGC !";
  $catnum = 0;
  $errstr ="Les requ&ecirc;tes NGC, IC et Messier sont seules accept&eacute;es.";
};

if ($catnum) {
  # Get the NGC 2000 data..
  $ngc2000line = `grep "^$catnum" $ngcfile`;
  $ngc2000line =~
    /.{6}(.{3}).(.{2}).(.{4}).{2}(.{3}).(.{2}).(.).{2}(.{3})(.{6}).{2}(.{5}).(.*)/;
  $type=$1; $RA=$2 . " : " . $3; $Dec=$4 . " : " . $5;  $Mod=$6;
  $con=$7; $Dim=$8; $Mag=$9; $Desc=$10;
  # Is there a caption ?
  $pixline = `grep "^$catnum:" $pixfile`;
  chomp($pixline);
  if (  $pixline =~ s/^[^:]*: //) {
    if (($pixline =~ /(.) ([-\w]{1,3}) (.*) "(.*)"/) ||
       ($pixline =~ /(.) ([-\w]{1,3}) (.*)/)) {
    $display=$1;
    $own=$2;
    $pix=$3;
    $pixcap=$4;
    }
  };
  # Is there a caption ? 
  $capline = `grep "^$catnum:" $capfile`;
  if ($capline =~ /.*: (.*)/) {$caption = $1;};
};

if (!$Desc) {$Desc="None"}; # No description
if ($mnum) {
  # Get Messier data
  if ($mnum==102) {$lin = "102?"} else {
    $lin = ('0' x (3-length($mnum))) . $mnum;
  };
  $mdataline = `grep "^$lin" $mdatafile`;
  chomp ($mdataline);
  $mdataline =~
    /.{27}(.{6}).{10}(..).(.{4}).(...).(..).(.{4}).(.{9}).(.{5,10})(.*)/;
  $Dist=$8; $MDesc=$9;
  if (!$catnum) {$con=$1; $RA=$2 . " : " . $3; $Dec=$4 . " : " . $5};
  if (!$Mag) {$Mag=$6};
  if (!$Dim) {$Dim=$7};
  if ($mnum==40) {$type="D*"}; #M40
  if ($mnum==45) {$type="C+N"}; #M45
  $Desc =~ s/;.*//;    # correct incomplete NGC 2000 data for M number
  if ($mnum==24) {
    $Desc = $Desc . "; dans M24";
  } elsif ($mnum==102) {
    $Desc = $Desc . "; = M102 (?)";
  } elsif ($mnum!=51) {
    $Desc = $Desc . "; = M$mnum";
  }
}

if ($cat ne "" ) {
  # Get cross references
  if ( $cat eq "M" ) { $cnum = "^" . "." x 25 . $cnum } else
    { $cnum = "^" . "." x 31 . $cnum } ;

  open CREFFILE , "grep '$cnum' $creffile |" ;
  @creflines = <CREFFILE> ;
  close CREFFILE ;
  if ( $#creflines == 0 ) {
    $crefline = $creflines[0]
  } else {
    ( $crefline ) = grep "$cnum " , @creflines 
  }

  # $crefline = `grep "$cnum" $creffile` ;
  ($CRm, $CRngcic, $CRgc, $CRh33, $CRh47, $CRwh, $CRlac, $CRdun, $CRrest, ) =
    $crefline =~
      /^.{25}(.{4})..(.{8})(.{9})(.{8})(.{8})(.{9})(.{9})..(.{9})(.*)$/;
  $CRm     =~ s/^ *//; $CRm     =~ s/ *$//;
  $CRngcic =~ s/^ *//; $CRngcic =~ s/ *$//;
  $CRgc    =~ s/^ *//; $CRgc    =~ s/ *$//;
  $CRh33   =~ s/^ *//; $CRh33   =~ s/ *$//;
  $CRh47   =~ s/^ *//; $CRh47   =~ s/ *$//;
  $CRwh    =~ s/^ *//; $CRwh    =~ s/ *$//;
  $CRlac   =~ s/^ *//; $CRlac   =~ s/ *$//;
  $CRdun   =~ s/^ *//; $CRdun   =~ s/ *$//;
  $CRrest  =~ s/^ *//; $CRrest  =~ s/ *$//;
  if ( $CRrest ne "" ) { $CRrest .=  "." }

  if (($CRm ne "") and ($CRm ne "M" . " " x (3-length($mnum)) . $mnum)) {
    $title = "* WARNING *" ;
    $errstr = "Messier number in Cross Ref File ($creffile): $CRm\n" .
      "doesn't match Messier number: $mnum" ;
    $cat = "";
  }

  if (($CRngcic ne "") and ! ($CRngcic =~ s/[NI] //) and ($CRngcic != $catnum )) {
    $title = "* WARNING *" ;
    $errstr = "NGC/IC number in Cross Ref File ($creffile): $CRngcic\n" .
      "doesn't match NGC/IC number: $catnum" ;
    $cat = "";
  }

  $CRefs = join ", ", $CRgc, $CRh33, $CRh47, $CRwh, $CRlac, $CRdun;
  $CRefs =~ s/(, )+/, /g; $CRefs =~ s/^, //; $CRefs =~ s/, $//;
  if ( $CRefs ne "" ) {
    $CRefs .= ".\n" . $CRrest ; 
  } else {
    $CRefs = $CRrest ;
  }
}

$con =~ s/ //g;
$type =~ s/ //g;
$con=$constell{$con};
$type=$types{$type};

if (!($cat eq "")) {
  $field=15;
  if ($Dim > 12.0) {
    if ($Dim < 50.0) {$field = 1.2*$Dim; } else { $field = 60; };
  }
  $tit = $title; $tit =~ s/ /!/g;
  $dssquery = "obj=$tit&r=$RA&d=$Dec&e=J2000&h=$field&w=$field&f=GIF&c=none&v=phase2_gsc2";
  $dssquery =~ s/ //g;
}

print "Content-type: text/html\n\n";
printf "<html><head>\n<title>%15s</title>\n\n", $title;
print '<link rev="owner" title="Contact Maintainer" href="http://www.seds.org/~spider/Contact/spider.html">',
  "\n" , '<meta name="author" content="Hartmut Frommert">' , "\n" ,
  '<meta name="distribution" value="global">' , "\n</head>\n\n<body>\n\n";
if ($mnum) {
  if ($mnum==102) { $mpic = "ngc5866" ; } else { $mpic = "m$mnum" ; } ;
  print '<a href="' , $mpixdir , $mpic , '.jpg">' ,  "\n" ,
        '<img src="', $mpixdir , $mpic , '.jpg" alt="[M' , $mnum , 
        ' image]" align=right>' , "</a>\n" ;
} elsif ($pix) {
  if ($display eq "r") {$display='align=right';$br=''}
  else {$display='';$br='<br>'};
  print '<a href="', $pix, '">', "\n<img ", $display, ' src="', $pix, 
        '" alt="[', $title, ' image]"></a>', $br, "\n";
  if ($pixcap) {print $pixcap, "\n";};
};
printf "<h2>%15s</h2>\n\n", $title;

if ($cat eq "") { # error
  print "Entr&eacute;e erron&eacute;e !\n$errstr\n<p>\n\n";
} else { # NGC, IC, M
  print "$type dans $con\n<p>\n\n";
  if ($MDesc) {print "<b>$MDesc</b>\n<p>\n\n";};
  print "<b>Description de Dreyer</b>:\n<tt>$Desc</tt>\n<p>\n\n";
  if ( $CRefs ne "" ) {
    print "<b><a href=\"cid.html\">Identifications crois&eacute;es</a></b>: $CRefs\n<p>\n" ;
  }
  if ($mnum) {
#
#    print "\n<p>\n<b>Testing !</b><br>\n  mdataline: $mdataline\n<p>\n\n";
#    print "RA: $RA, Dec: $Dec\n<p>\n\n";
#  
    print '<li><a href="' , $mcapdir , 'm', ('0' x (3-length($mnum))), $mnum,
          '.html">Voir aussi ', $title, " (SEDS)</a>\n<p>\n\n";
  } elsif ($caption) {
    chomp($caption);
    $caption = '<li><a href="' . $caption . '">Voir aussi ' . $title . "</a>\n<p>\n\n";
    print $caption;
  };
  print "\n<br clear=all>\n";
  print "<table border=2 cellpadding=5 cellspacing=0>\n";
  print "  <tr><th align=left>Ascension droite: </th><td>", $RA, " (heures : minutes)<br></td></tr>\n";
  print "  <tr><th align=left>D&eacute;clinaison:    </th><td>", $Dec, " (degr&eacute;s : minutes)<br></td></tr>\n";
  if ($Dist) {
    print "  <tr><th align=left>Distance: </th><td>", $Dist, " (* 1000 Ann&eacute;es-lumi&egrave;re)<br></td></tr>\n";
  };
  print "  <tr><th align=left>Magnitude apparente: </th><td>", $Mag, "<br></td></tr>\n";
  print "  <tr><th align=left>Diam&egrave;tre apparent: </th><td>", $Dim, " (minutes d'arc)<br></td></tr>\n";
  print "</table>\n<p>\n\n";

  print '<a href="ngc_fr.html"><img alt="" src="ngc.jpg" border=0> NGC Home</a>', "\n";

  if ($catnum) {
    if ($catnum =~ /^i/i) {
      $inum = $catnum; $inum =~ s/i//i;
      if ($inum==1) {$num=$ICmax} else {$num=$inum-1};
      print '<img src="left.jpg" alt="<"><a href="ngc_fr.cgi?i', $num, '"> IC ', $num, "</a> |\n";
      if ($inum==$ICmax) {$num=1} else {$num=$inum+1};
      print '<a href="ngc_fr.cgi?i', $num, '">IC ', $num, '</a> <img src="right.jpg" alt=">">', "\n";
    } else {
      if ($catnum==1) {$num=$NGCmax} else {$num=$catnum-1};
      print '<img src="left.jpg" alt="<"> <a href="ngc_fr.cgi?', $num, '"> NGC ', $num, "</a> |\n"; 
      if ($catnum==$NGCmax) {$num=1} else {$num=$catnum+1};
      print '<a href="ngc_fr.cgi?', $num, '">NGC ', $num, '</a> <img src="right.jpg" alt=">">', "\n";
    };
  };  
  if ($mnum) {
    if ($mnum==1) {$num=$Mmax} else {$num=$mnum-1};
    print '<img src="left.jpg" alt="<"> <a href="ngc_fr.cgi?m', $num, '">M', $num, "</a> |\n";
    if ($mnum==$Mmax) {$num=1} else {$num=$mnum+1};
    print '<a href="ngc_fr.cgi?m', $num, '">M', $num, '</a> <img src="right.jpg" alt=">">', "\n";
  };
  print "<p>\n\n", '<li><a href="ngcdss_fr.cgi?', $dssquery, '">Image du catalogue Digital Sky Survey</a>', "\n\n";
  
  if ($catnum) {
    $inum = $catnum;
    if ( $catnum =~ /^i/i ) { # IC  
      $inum =~ s/^i */IC/i;
    } else { # NGC
      $inum =~ s/^ */NGC/i;
    }
    print '<li><a href="ngcic.cgi?' , $inum , '">Base de donn&eacute;e NGC/IC</a>' , "\n\n" ;
  }

  if ($mnum) {
    $ned = "M$mnum";
  } else {
    $ned = $catnum; $ned =~ s/[^0-9]//g; $ned = "$cat$ned";
  }
  print '<li><a href="http://simbad.u-strasbg.fr/sim-id.pl?Ident=' , $ned ,
    '">Base de donn&eacute;es SIMBAD</a>' , "\n\n";
#  print '<li><a href="http://archive.stsci.edu/cgi-bin/hst?action=search&resolver=SIMBAD&radius=10.0&sort1=ang_sep&max_rows=500&target=' ,
  print '<li><a href="http://archive.stsci.edu/hst/search.php?action=Search&resolver=SIMBAD&radius=10.0&ordercolumn1=ang_sep&max_records=500&outputformat=HTML_Table&sci_aec=S&showquery=on&target=' ,
    $ned, "\">Images d'archive du HST</a>\n\n";

#  if ($type eq "Galaxy") {
  if ($mnum) {
    if ($mnum = 102) {
      $ned = "UGC+9723";
    } else {
      $ned = "M+$mnum";
    }
  } else {
    $ned = $catnum; $ned =~ s/[^0-9]//g; $ned = "$cat+$ned";
  }
  print '<li><a href="http://nedwww.ipac.caltech.edu/cgi-bin/nph-objsearch?objname=',
    $ned, '&extend=yes&out_equinox=J2000.0">Base de donn&eacute;es NED</a>', "\n\n";
#  }

  if ($mnum) {
    $ned = "M+$mnum";
  } else {
    $ned = $catnum; $ned =~ s/[^0-9]//g; $ned = "$cat+$ned";
  }
  print '<li><a href="http://adsabs.harvard.edu/cgi-bin/abs_connect?db_key=AST&sim_query=YES&object=',
    $ned, '">Publications et References (ADS)</a>' , "\n\n" ;

  if ($cat eq "M") {
    $catalog = "Messier" ;
    $ids = $mnum ;
  } else {
    $catalog = $cat ;
    $ids = $catnum; $ids =~ s/[^0-9]//g;
  }
  print '<li><a href="http://www.visualdeepsky.org/search.cgi?' ,
    "catalogs=$catalog&ids=$ids\">Rapports d'Observation</a> (IAAC Netastrocatalog)";

};

print <<'EofF';
<hr>

<b>Nouvelle recherche</b>:
<p>

Vous devez entrer le num&eacute;ro NGC (soit en tant que nombre, soit pr&eacute;c&eacute;d&eacute; de 'N' 
ou 'NGC'), ou le num&eacute;ro IC pr&eacute;c&eacute;d&eacute; de 'IC' ou 'I', ou
encore le num&eacute;ro Messier pr&eacute;c&eacute;d&eacute; de 'M'.

<isindex action="ngc_fr.cgi" prompt="Entrez le num&eacute;ro du catalogue: ">

<p>
<a href="http://www.seds.org/~spider/">Hartmut Frommert</a>
<a href="../Contact/spider.html">[contact]</a>
<p>
<a href="../index.html"><img src="../Jco/spider.ico.jpg" alt="[Spider] @"></a>
<a href="http://www.seds.org/"><img src="../Jco/seds.jpg" alt="[SEDS]"></a>

<p>
Traduction fran&ccedil;aise:<br>
Bernard Tr&eacute;z&eacute;guet<br>
Le 5 Mai 2001<br>

</body></html>
EofF
