#!/usr/bin/perl -w --
#
# Viewer for revngc and revic catalogs
# by Hartmut Frommert

my (
  $title ,
  $query ,
  $cat ,
  $catnum ,
  $catfile ,
  $revngcfile ,
  $revicfile ,
  %constellation ,
  %status ,

  $entry ,
  $count ,
  $dreyer ,
  $lineno ,
  $state ,
  $prec ,
  $con ,
  $ra , $dec ,
  $bmag , $vmag , $sb ,
  $exta , $extb , $pa ,
  $type ,
  $pgcno ,
  $id1 , $id2 , $id3 ,
  $remark,

  %idtitle ,
  %descline ,
  %posline ,
  %idsline ,
  $id ,
  );

$revngcfile = 'revngc.txt' ;
$revicfile = 'revic.txt' ;

%constellation = (
  'AND' => 'Andromeda' ,
  'ANT' => 'Antlia' ,
  'APS' => 'Apus' ,
  'AQL' => 'Aquila' ,
  'AQR' => 'Aquarius' ,
  'ARA' => 'Ara' ,
  'ARI' => 'Aries' ,
  'AUR' => 'Auriga' ,
  'BOO' => 'Bootes' ,
  'CAE' => 'Caelum' ,
  'CAM' => 'Camelopardalis' ,
  'CAP' => 'Capricornus' ,
  'CAR' => 'Carina' ,
  'CAS' => 'Cassiopeia' ,
  'CEN' => 'Centaurus' ,
  'CEP' => 'Cepheus' ,
  'CET' => 'Cetus' ,
  'CHA' => 'Chamaeleon' ,
  'CIR' => 'Circinus' ,
  'CMA' => 'Canis Major' ,
  'CMI' => 'Canis Minor' ,
  'CNC' => 'Cancer' ,
  'COL' => 'Columba' ,
  'COM' => 'Coma Berenicis' ,
  'CRA' => 'Corona Australis' ,
  'CRB' => 'Corona Borealis' ,
  'CRT' => 'Crater' ,
  'CRU' => 'Crux' ,
  'CRV' => 'Corvus' ,
  'CVN' => 'Canes Venatici' ,
  'CYG' => 'Cygnus' ,
  'DEL' => 'Delphinus' ,
  'DOR' => 'Dorado' ,
  'DRA' => 'Draco' ,
  'EQU' => 'Equuleus' ,
  'ERI' => 'Eridanus' ,
  'FOR' => 'Fornax' ,
  'GEM' => 'Gemini' ,
  'GRU' => 'Grus' ,
  'HER' => 'Hercules' ,
  'HOR' => 'Horologium' ,
  'HYA' => 'Hydra' ,
  'HYI' => 'Hydrus' ,
  'IND' => 'Indus' ,
  'LAC' => 'Lacerta' ,
  'LEO' => 'Leo' ,
  'LEP' => 'Lepus' ,
  'LIB' => 'Libra' ,
  'LMI' => 'Leo Minor' ,
  'LUP' => 'Lupus' ,
  'LYN' => 'Lynx' ,
  'LYR' => 'Lyra' ,
  'MEN' => 'Mensa' ,
  'MIC' => 'Microscopium' ,
  'MON' => 'Monoceros' ,
  'MUS' => 'Musca' ,
  'NOR' => 'Norma' ,
  'OCT' => 'Octans' ,
  'OPH' => 'Ophiuchus' ,
  'ORI' => 'Orion' ,
  'PAV' => 'Pavo' ,
  'PEG' => 'Pegasus' ,
  'PER' => 'Perseus' ,
  'PHE' => 'Phoenix' ,
  'PIC' => 'Pictor' ,
  'PSA' => 'Piscis Austrinus' ,
  'PSC' => 'Pisces' ,
  'PUP' => 'Puppis' ,
  'PYX' => 'Pyxis' ,
  'PsA' => 'Piscis Austrinus' ,
  'RET' => 'Reticulum' ,
  'SCL' => 'Sculptor' ,
  'SCO' => 'Scorpius' ,
  'SCT' => 'Scutum' ,
  'SER' => 'Serpens' ,
  'SEX' => 'Sextans' ,
  'SGE' => 'Sagitta' ,
  'SGR' => 'Sagittarius' ,
  'TAU' => 'Taurus' ,
  'TEL' => 'Telescopium' ,
  'TRA' => 'Triangulum Australis' ,
  'TRI' => 'Triangulum' ,
  'TUC' => 'Tucana' ,
  'UMA' => 'Ursa Major' ,
  'UMI' => 'Ursa Minor' ,
  'VEL' => 'Vela' ,
  'VIR' => 'Virgo' ,
  'VOL' => 'Volans' ,
  'VUL' => 'Vulpecula' ,
) ;

%status = (
   '1' => 'Galaxy' ,
   '2' => 'Diffuse Nebula or Supernova Remnant' ,
   '3' => 'Planetary Nabula' ,
   '4' => 'Open Cluster' ,
   '5' => 'Globular Cluster' ,
   '6' => 'Part of Galaxy' ,
   '7' => 'Duplicate NGC/IC Object' ,
   '8' => 'Duplicate of NGC Object' ,
   '9' => 'Star(s)' ,
  '10' => 'Object Not Found' ,
) ;

sub err {
  my ( $basename , ) ;
  my ( $errtext , ) = @_ ;
  $basename = $0 ; $basename =~ s/^.*\/// ;
  $title = sprintf 'Error in %s' , $basename ;
  printf "Content-Type: text/html\n\n<html><head>\n<title>$title</title>\n</head>\n\n" ;
  printf "<body>\n<center>\n<h2>$title</h2>\n</center>\n\n" ;
  printf "<b>Error:</b> %s.\n\n" , $errtext ;
  printf "</body></html>\n";
  exit(1);
}

sub objectstate {
  my ( $state , ) = @_ ;

  if ( $state ne '' ) { $state = $status{ $state } ; } ;
  return $state ;
}

sub gettype {
  my ( $type , ) = @_ ;

  if ( $type ne '' ) { 
    return sprintf ', type %s,' , $type ;
  }
}

sub addobject {
  my ( $cat , $entry , $count , $dreyer , $state , $con , $ra , $dec , $bmag ,
       $vmag , $sb , $exta , $extb , $pa , $type , $pgcno , $id1 , $id2 , $id3 ,
       $remark , ) = @_ ;
  my ( $id , $entryno , $entryext , ) ;

  ( $entryno , $entryext , ) = $entry =~ /^([0-9]*)(.*)/ ;

  if ( $count ne "" ) { $entryext = sprintf '%s-%s' , $entryext , $count ; }
  
  $id = sprintf '%s %4d%s' , $cat , $entryno , $entryext ;

  if ( $dreyer eq "" ) {
    $idtitle{ $id } = $id ;
  } else { # $dreyer eq ""
    $idtitle{ $id } = sprintf "%s (%s)" , $id , $dreyer ;
  }

  $descline{ $id } = sprintf "%s%s in %s\n" , objectstate( $state ) ,
    gettype( $type ) , $constellation{ $con } ;
  $posline{ $id } = sprintf
    "Right Ascension (2000.0): %s (h:m:s)<br>\nDeclination (2000.0): %s (deg:m:s)\n" ,
    $ra , $dec ; 

  # create magline
  if ( $bmag ne "" ) {  
    if ( $vmag ne "" ) {
      if ( $sb ne "" ) {       # $bmag, $vmag, $sb
        $magline{ $id } = sprintf "m_b: %s (mag) , m_v: %s (mag) , SB: %s (mag per square arcmin)\n" , $bmag , $vmag , $sb ;
      } else {                 # $bmag, $vmag
        $magline{ $id } = sprintf "m_b: %s (mag) , m_v: %s (mag)\n" , $bmag , $vmag ;
      }
    } elsif ( $sb ne "" ) {    # $bmag, $sb
      $magline{ $id } = sprintf "m_b: %s (mag) , SB: %s (mag per square arcmin)\n" , $bmag , $sb ;
    } else {                   # $bmag
      $magline{ $id } = sprintf "m_b: %s (mag)\n" , $bmag ;
    }
  } elsif ( $vmag ne "" ) {
    if ( $sb ne "" ) {         # $vmag, $sb
      $magline{ $id } = sprintf "m_v: %s (mag) , SB: %s (mag per square arcmin)\n" , $vmag , $sb ;
    } else {                   # $vmag
      $magline{ $id } = sprintf "m_v: %s (mag)\n" , $vmag ;
    }
  } elsif ( $sb ne "" ) {      # $sb
    $magline{ $id } = sprintf " SB: %s (mag per square arcmin)\n" , $sb ;
  } else {                     # None
    $magline{ $id } = '' ;
  }

  # create dimline
  if ( $exta ne "" ) {
    if ( $extb ne "" ) {
      if ( $pa ne "" ) {       # $exta , $extb , $pa
        $dimline{ $id } = sprintf "Dimension: %s x %s (arcmin) , PA: %s\n" , $exta , $extb , $pa ;
      } else {                 # $exta , $extb
        $dimline{ $id } = sprintf "Dimension: %s x %s (arcmin)\n" , $exta , $extb ;
      } 
    } elsif ( $pa ne "" ) {    # $exta , $pa
      $dimline{ $id } = sprintf "Dimension: %s (arcmin) , PA: %s\n" , $exta , $pa ;
    } else {                   # $exta
      $dimline{ $id } = sprintf "Dimension: %s (arcmin)\n" , $exta ;
    }
  } elsif ( $extb ne "" ) {
    if ( $pa ne "" ) {         # $extb , $pa
      $dimline{ $id } = sprintf "Dimension: %s (arcmin) , PA: %s\n" , $extb , $pa ;
    } else {                   # $extb
      $dimline{ $id } = sprintf "Dimension: %s (arcmin)\n" , $extb ;
    }
  } elsif ( $pa ne "" ) {      # $pa
    $dimline{ $id } = sprintf "PA: %s\n" , $pa ;
  } else {                     # None
    $dimline{ $id } = '' ;
  }

  # create idsline
  if ( $id1 ne '' ) {
    $idsline{ $id } = sprintf '%s' , $id1 ;
    if ( $id2 ne '' ) {
      $idsline{ $id } = sprintf '%s, %s' , $idsline{ $id } , $id2 ;
      if ( $id3  ne '' ) {
        $idsline{ $id } = sprintf '%s, %s' , $idsline{ $id } , $id3 ;
      }
    }
  }
  if ( $pgcno ne '' ) {
    if ( $idsline{ $id } ne '' ) { 
      $idsline{ $id } = sprintf '%s, PGC %s' , $idsline{ $id } , $pgcno ;
    } else { $idsline{ $id } = sprintf 'PGC %s' , $pgcno ; }
  }
  if ( $remark ne '' ) {
    if ( $idsline{ $id } ne '' ) { 
      $idsline{ $id } = sprintf '%s, %s' , $idsline{ $id } , $remark ;
    } else { $idsline{ $id } = $remark ; }
  }
  if ( $idsline{ $id } ne '' ) {
    $idsline{ $id } = sprintf 'Cross Identifications: %s' , $idsline{ $id } ;
  }
#print "addobject args: @_\n" ;
#printf "Id: %s; State: %s; Desc: %s\n" , $id , $state , $descline{ $id } ;
# $count , $dreyer , 
}

sub addobjectids {
  my ( $cat, $entry, $id1, $id2, $id3, $remark ) = @_ ;
  my ( $id , $entryno , $entryext , ) ;

  ( $entryno , $entryext , ) = $entry =~ /^([1-9][0-9]*)(.*)/ ;
  
  $id = sprintf '%s %4d%s' , $cat , $entryno , $entryext ;

  $idsline{ $id } = sprintf '%s, %s' , $idsline{ $id } , $id1 ;
  if ( $id2 ne '' ) {
    $idsline{ $id } = sprintf '%s, %s' , $idsline{ $id } , $id2 ;
    if ( $id3  ne '' ) {
      $idsline{ $id } = sprintf '%s, %s' , $idsline{ $id } , $id3 ;
    }
  }
#print "addobjectids args: @_\n" ;
#print "Entry: $entry; EntryNo: $entryno ; EntryExt: $entryext\n" ;
}

$query = $ENV{ 'QUERY_STRING' } ;
$query =~ s/%20/ /g ;
$query =~ s/\+/ /g ;
$query =~ s/^ *//g ;

$query =~ tr/a-z/A-Z/ ;

if ( $query =~ /^(I|IC|N|NGC) */ ) {
  ( $cat , $catnum , ) = $query =~ /^(I|IC|N|NGC) *([1-9].*) *$/ ;
 # ( $cat , $catnum , ) = $query =~ /^(I|IC|N|NGC) *(.*)$/ ;
  $catnum =~ s/ *$// ;
} else {
  err 'Entries must be "NGC ####" or "IC ####"' ;
}

if ($cat eq 'N') { $cat = 'NGC' ; }
elsif ($cat eq 'I') { $cat = 'IC' ; }

if ($cat eq 'NGC' ) { $catfile = $revngcfile ; }
elsif ($cat eq 'IC') { $catfile = $revicfile ; }
else { err 'Unknown catalog: $cat!' ; }

( $catno , $catext , ) = $catnum =~ /^([1-9][0-9]*)(.*)$/ ;
$pattern = sprintf '%4d%s' , $catno , $catext ;

open CATFILE, $catfile ;
while ( <CATFILE> ) {
  if ( /^$pattern/ ) { 
    @lines = ( @lines , $_ ) ; 
  }
}
close CATFILE ;

# > IC  C D X S P CON    Rect (2000) Decl     Bmag   Vmag   SB     a      b   PA Type         PGC  ID1             ID2             ID3             Remarks                        
# >NGC  C D X S P CON    Rect (2000) Decl     Bmag   Vmag   SB     a      b   PA Type         PGC  ID1             ID2             ID3             Remarks                        
#-------------------------------
# >5866B      1 1 DRA  15 12 07.2 +55 47 07   15.2   14.6  16.2   2.70   1.9  20 SBd        54267  UGC 9769        MCG 9-25-34     ZWG 274.33      KUG 1510+559
# > 221   * 1 1 1 AND  00 42 41.8 +40 51 57    9.1    8.1  12.5   8.50   6.5 179 E2          2555  M 32            UGC 452         MCG 7-2-15      ZWG 535.16                      
# > 221     2                                                                                      IRAS00399+4035  ARAK 12         ARP 168     

foreach $line ( @lines ) {
  ( $entry , $count , $dreyer , $lineno , $state , $prec , $con , $ra , $dec , $bmag , $vmag , $sb , $exta , $extb , $pa , $type ,
    $pgcno , $id1 , $id2 , $id3 , $remark , ) =
    $line =~ /^(.{5})(.) (.)(..)(..) (.) (...)  (.{10}) (.{9})   (.{4})   (.{4})  (.{4}) (.{6}) (.{5}) (...) (.{8}) (.{7})  (.{14})  (.{14})  (.{14})  (.*) *$/ ;
    $entry =~ s/ //g ;
    $count =~ s/ //g ;
    $dreyer =~ s/ //g ;
    $lineno =~ s/ //g ;
    $state =~ s/^ *// ;
    $bmag =~ s/ //g ; $vmag =~ s/ //g ; $sb =~ s/ //g ; $exta =~ s/ //g ; $extb =~ s/ //g ; $pa =~ s/ //g ;
    $type =~ s/^ *// ; $type =~ s/ *$// ; $pgcno =~ s/ //g ;
    $id1 =~ s/^ *// ; $id1 =~ s/ *$// ; $id2 =~ s/^ *// ; $id2 =~ s/ *$// ; $id3 =~ s/^ *// ; $id3 =~ s/ *$// ;
    $remark =~ s/^ *// ; $remark =~ s/ *$// ;

    if ( ( $lineno eq '' ) or ( $lineno eq '1' ) ) {
      $ra =~ s/ /:/g ; $dec =~ s/ /:/g ;
      $bmag =~ s/ //g ; $vmag =~ s/ //g ; $sb =~ s/ //g ;
      $exta =~ s/ //g ; $extb =~ s/ //g ; $pa =~ s/ //g ;
      $type =~ s/^ *//; $type =~ s/ *$// ; $pgcno =~ s/ //g ;
      $ids = "$id1, $id2, $id3" ;
      addobject $cat , $entry , $count , $dreyer , $state ,  $con , $ra , $dec , $bmag , $vmag , $sb , $exta , $extb , $pa , $type ,
        $pgcno , $id1 , $id2 , $id3 , $remark ; 
    } else {
      addobjectids $cat , $entry , $id1 , $id2 , $id3 ;
    }
}

$title = sprintf 'Revised %s Data for %s %s' , $cat , $cat , $catnum ;

print <<EoT ;
Content-Type: text/html

<html><head>
<title>$title</title>
</head>

<body>
<center>
<h2>$title</h2>
</center>

Data from <a href="http://www.klima-luft.de/steinicke/">Wolfgang Steinicke</a>'s 
<a href="http://www.ngcic.org/steinicke/">Revised NGC and IC Catalog</a>
<p>
EoT

foreach $id ( sort keys ( %descline ) ) {
  printf "<b>Object: %s</b>\n<p>\n%s<br>\n%s<br>\n%s<br>\n%s<br>\n%s\n<p>\n" ,
    $idtitle { $id } , $descline{ $id } , $posline{ $id } , $magline{ $id } , $dimline{ $id } , $idsline { $id } ;
}

print <<EoF
<p>
<hr width="25%" align=left>
<b>Note</b>: (*) Dreyer's Object

<p>
<hr>

<ul>
<li><a href="http://www.ngcic.org/steinicke/">Revised NGC and IC Catalog</a>
<li><a href="ngcic.cgi?$cat$catnum">NGC/IC data for $cat $catnum</a>
<li><a href="ngc.cgi?$cat+$catnum">Back to $cat $catnum page</a>
</ul>

<hr>

<a href="http://www.seds.org/~spider/">Hartmut Frommert</a>
<a href="../Contact/spider.html">[contact]</a>
<p>
<a href="ngc.html"><img src="ngc.jpg" alt="[NGC Home]"></a>
<a href="http://www.seds.org/~spider/"><img src="../Jco/spider.ico.jpg" alt="[Spider]"></a>
<a href="http://www.seds.org/"><img src="../Jco/seds.jpg" alt="[SEDS]"></a>

</body></html>
EoF
